/** OK, 0.584, 22 mb (18 mb if x32) */

#include <cassert>
#include <cstdio>
#include <cstdlib>
#include <algorithm>

using namespace std;

#define MP make_pair
#define Y first
#define X second
#define RAND ((rand() << 15) | rand())

typedef pair <int, int> PII;
                        
const int maxn = 2e5 + 1;

int res = 0, n, w, h, clen[maxn];
PII rp, p[maxn];

#pragma pack(4)
struct segment
{
  int i, x, len;
  segment() {}
  segment( int _i, int _x, int _len ) { i = _i, x = _x, len = _len; }
};
struct tree
{
  int y, mlen;
  segment s;
  tree *l, *r;
};

const int maxt = 4 * maxn;
tree tmem[maxt];
int tpos;

inline void Calc( tree *t )
{
  t->mlen = t->s.len;
  if (t->l && t->mlen < t->l->mlen) t->mlen = t->l->mlen;
  if (t->r && t->mlen < t->r->mlen) t->mlen = t->r->mlen;
}

tree *NewT( segment s, int y, tree *l, tree *r )
{
  assert(tpos <= maxt);
  tree *res = &tmem[tpos++];
  res->l = l, res->r = r, res->s = s, res->y = y;
  Calc(res);
  return res;
}

inline bool operator == ( const segment &a, const segment &b ) { return MP(a.x, MP(a.len, a.i)) == MP(b.x, MP(b.len, b.i)); }
inline bool operator <  ( const segment &a, const segment &b ) { return MP(a.x, MP(a.len, a.i)) <  MP(b.x, MP(b.len, b.i)); }

inline void Split( tree *t, tree **l, tree **r, segment s )
{
  if (!t)
    *l = *r = 0;
  else if (s < t->s)
    Split(t->l, l, &t->l, s), Calc(*r = t);
  else
    Split(t->r, &t->r, r, s), Calc(*l = t);
}

inline void Add( tree **t, segment s, int y )
{
  clen[s.i] = s.len;
  if (*t == 0)
    *t = NewT(s, y, 0, 0);
  else if ((*t)->y < y)
    Add(s < (*t)->s ? &(*t)->l : &(*t)->r, s, y), Calc(*t);
  else
  {
    tree *l, *r;
    Split(*t, &l, &r, s);
    *t = NewT(s, y, l, r);
  }
}

inline void Merge( tree **t, tree *l, tree *r )
{
  if (!l)
    *t = r;
  else if (!r)
    *t = l;
  else if (l->y < r->y)
    Merge(&l->r, l->r, r), Calc(*t = l);
  else
    Merge(&r->l, l, r->l), Calc(*t = r);
}

inline void Del( tree **t, segment s )
{
  if ((*t)->s == s)
    Merge(t, (*t)->l, (*t)->r);
  else 
    Del(s < (*t)->s ? &(*t)->l : &(*t)->r, s), Calc(*t);
}

inline tree *Find( tree *t, int x )
{
  if (!t)
    return t;
  if (t->s.x <= x)
  {
    tree *p = Find(t->r, x);
    return (!p || p->s.x > x) ? t : p;
  }
  return Find(t->l, x);
}

inline tree *FindI( tree *t, segment s )
{
  if (!t || t->s.i == s.i)
    return t;
  if (t->l && s < t->s)
    return FindI(t->l, s);
  tree *p = FindI(t->r, s);
  return (!p || p->s.x > s.x) ? t : p;
}

inline tree *Get( tree *t, int ma )
{
  if (t->mlen < ma)
    if (t->l && t->l->mlen == t->mlen)
      return Get(t->l, ma);
    else if (t->s.len == t->mlen)
      return t;
    else
      return Get(t->r, ma);
  else
    if (t->l && t->l->mlen >= ma)
      return Get(t->l, ma);
    else if (t->s.len >= ma)
      return t;
    else
      return Get(t->r, ma);
}

void Solve( void )
{
  int i = 0, j = 0, ymi = 0, yma = p[0].Y, len;
  tree *tr = 0, *tmp;
  tpos = 0;

  p[n].Y = h;
  p[n + 1].X = 0;
  Add(&tr, segment(n + 1, 0, w), RAND);

  tmp = Get(tr, yma - ymi);
  int ma = tmp->s.len, xma = tmp->s.x;
  if (ma > yma - ymi)
    ma = yma - ymi;
  if (ma > res || (ma == res && rp > MP(xma, ymi)))
    res = ma, rp = MP(xma, ymi);
  do
  {
    tmp = Find(tr, p[j].X);
    Del(&tr, tmp->s);
    Add(&tr, segment(j, p[j].X, tmp->s.len - (p[j].X - tmp->s.x)), RAND);
    tmp->s.len = p[j].X - tmp->s.x;
    Add(&tr, tmp->s, RAND);
    yma = p[++j].Y;

    while (1)
    {
      tmp = Get(tr, yma - ymi);
      int ma = tmp->s.len, xma = tmp->s.x;
      if (ma > yma - ymi)
        ma = yma - ymi;
      if (ma > res || (ma == res && rp > MP(xma, ymi)))
        res = ma, rp = MP(xma, ymi);
      if (tr->mlen > yma - ymi || (!tr->l && !tr->r))
        break;

      tmp = FindI(tr, segment(i, p[i].X, clen[i])), len = tmp->s.len;
      Del(&tr, tmp->s);
      tmp = FindI(tr, segment(i, p[i].X, clen[i]));
      Del(&tr, tmp->s);
      tmp->s.len += len;
      Add(&tr, tmp->s, RAND);
      ymi = p[i++].Y;
    }
  } while (j < n);
}

int main( void )
{
  assert(freopen("roi.in", "rt", stdin));
  assert(freopen("roi.out", "wt", stdout));

  scanf("%d%d%d", &w, &h, &n);
  for (int i = 0; i < n; i++)
    scanf("%d%d", &p[i].X, &p[i].Y);
  sort(p, p + n);

  Solve();
  printf("%d\n%d %d\n", res, rp.first, rp.second);
  return 0;
}
