#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <cassert>

#include <iostream>
#include <iomanip>
#include <string>
#include <vector>
#include <set>
#include <map>
#include <deque>
#include <queue>
#include <stack>
#include <algorithm>


#define forn(i, n) for(int i = 0; i < int(n); i++)
#define mp make_pair
#define pb push_back
#define foreach(it, v) for(__typeof(v.begin()) it = v.begin(); it != v.end(); ++it)

using namespace std;

typedef long long ll;
typedef pair < int, int > pair_t;

const int MAXN = 200010;
const int BUBEN = 300;

int n, m;
int arr[MAXN], newArr[MAXN];
char query[8];

struct Segment {
	int l, r;
	vector < pair_t > sorted;
	Segment(int l, int r) : l(l), r(r) {}
	Segment(int l, int r, int value) : l(l), r(r), sorted(1) { sorted[0] = mp(value, l); }	
	int size() const { return r - l; }
	
	void reserve() {
		sorted.reserve(size());
	}
	
	void rebuild() {
		reserve();
		for(int i = 0; i < size(); i++) {
			sorted.pb(mp(arr[l + i], l + i));
		}
		sort(sorted.begin(), sorted.end());
	}
	
	int get(int x) const {
		return distance(sorted.begin(), lower_bound(sorted.begin(), sorted.end(), mp(x + 1, 0)));
	}
};

vector < Segment * > segments;

void rebuild() {
	if(!segments.empty()) {
		m = 0;
		foreach(it, segments) {
			Segment * s = *it;
			copy(arr + s->l, arr + s->r, newArr + m);
			m += s->size();
			delete s;
		}
		segments.clear();
		copy(newArr, newArr + m, arr);
		n = m;
	}
	for(int i = 0; i < m; i += BUBEN) {
		int l = i, r = min(n, i + BUBEN);
		Segment * newSegment = new Segment(l, r);
		newSegment->rebuild();
		segments.push_back(newSegment);
	}
}

void printArray() {
	printf("DEBUG \n");
	foreach(it, segments) {
		Segment * s = *it;
		for(int i = s->l; i < s->r; i++)
			printf("%d ", arr[i]);
	}
	printf("\n");
}

int split(int index) {
	if(index == m)
		return segments.size();

	int skipped = 0, segmentIndex = 0;
	
	while(skipped + segments[segmentIndex]->size() <= index) {
		skipped += segments[segmentIndex]->size();
		++segmentIndex;
	}
	
	if(skipped == index)
		return segmentIndex;
	
	Segment * segmentToSplit = segments[segmentIndex];
	
	int leftSize = index - skipped;
	
	Segment * leftSegment = new Segment(segmentToSplit->l, segmentToSplit->l + leftSize);
	Segment * rightSegment = new Segment(segmentToSplit->l + leftSize, segmentToSplit->r);
	
	leftSegment->reserve();
	rightSegment->reserve();
	
	for(int i = 0; i < segmentToSplit->size(); i++) {
		pair_t p = segmentToSplit->sorted[i];
		if(p.second < leftSegment->r)
			leftSegment->sorted.pb(p);
		else
			rightSegment->sorted.pb(p);
	}
	
	segments.erase(segments.begin() + segmentIndex);
	segments.insert(segments.begin() + segmentIndex, rightSegment);
	segments.insert(segments.begin() + segmentIndex, leftSegment);
	
	delete segmentToSplit;
	
	return segmentIndex + 1;
}

void insert(int index, int value) {
	arr[n] = value;
	int splitIndex = split(index);
	segments.insert(segments.begin() + splitIndex, new Segment(n, n + 1, value));
	++n;
	++m;
}

void erase(int index) {
	int splitIndex = split(index);
	split(index + 1);
//	printArray();
	segments.erase(segments.begin() + splitIndex);
	//printArray();
	--m;
}

ll get(int l, int r, int x) {
	l = split(l);
	r = split(r + 1);
	ll result = 0;
	while(l < r) {
		result += segments[l++]->get(x);
	}
	return result;
}

int main() {
	scanf("%d", &n);
	for(int i = 0; i < n; i++)
		scanf("%d", &arr[i]);

	m = n;
		rebuild();
	
	while(scanf("%s", query) == 1) {
		char q = query[0];
		if(q == '+') {
			int index, value;
			scanf("%d%d", &index, &value);
			insert(index, value);
		} else if(q == '-') {
			int index;
			scanf("%d", &index);
			erase(index);
		} else {
			int l, r, x;
			scanf("%d%d%d", &l, &r, &x);
			ll result = get(l, r, x);
			printf("%lld\n", result);
		}		
		
		if(segments.size() >= 3 * BUBEN) {
			rebuild();
		}
	}
	return 0;
}