#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <cassert>

#include <iostream>
#include <iomanip>
#include <string>
#include <vector>
#include <set>
#include <map>
#include <deque>
#include <queue>
#include <stack>
#include <algorithm>


#define forn(i, n) for(int i = 0; i < int(n); i++)
#define mp make_pair
#define pb push_back
#define foreach(it, v) for(__typeof(v.begin()) it = v.begin(); it != v.end(); ++it)
#define all(v) v.begin(), v.end()

using namespace std;

typedef long long ll;

typedef pair < int, int > tree_t;
typedef pair < int, pair < int, int > > result_t;
typedef pair < int, int > event_t;

const result_t NONE = mp(1e9 + 100, mp(1e9 + 100, 1e9 + 100));

struct EmptySegment {
	int length;
	int result, resultIndex;
	int left, leftIndex;
	int right, rightIndex;
	
	EmptySegment(int length) : length(length), result(0), resultIndex(1e9), left(0), leftIndex(1e9), right(0), rightIndex(1e9) {}
	EmptySegment(int length, int index) : length(length), result(length), resultIndex(index), left(length), leftIndex(index), right(length), rightIndex(index) {}
	
	static EmptySegment merge(const EmptySegment & lSeg, const EmptySegment & rSeg) {
		EmptySegment result(lSeg.length + rSeg.length);
		
		{
			result.result = lSeg.result;
			result.resultIndex = lSeg.resultIndex;
			
			if(rSeg.result > result.result) {
				result.result = rSeg.result;
				result.resultIndex = rSeg.resultIndex;
			}
			
			if(lSeg.right + rSeg.left > result.result || (lSeg.right + rSeg.left == result.result && lSeg.rightIndex < result.resultIndex)) {
				result.result = lSeg.right + rSeg.left;
				result.resultIndex = lSeg.rightIndex;
			}
		}
		
		result.left = lSeg.left;
		result.leftIndex = lSeg.leftIndex;
		if(lSeg.left == lSeg.length)
			result.left += rSeg.left;
			
		result.right = rSeg.right;
		result.rightIndex = rSeg.rightIndex;
		if(rSeg.right == rSeg.length && lSeg.right > 0) {
			result.right += lSeg.right;
			result.rightIndex = lSeg.rightIndex;
		}
		
		return result;
	}
	
	void print() const {
		//cerr << "SEGMENT: " << result << " " << resultIndex << " " << left << " " << leftIndex << " " << right << " " << rightIndex << endl;
	}
};

struct SegTreeNode {
	SegTreeNode * l, *r;
	int level;
	EmptySegment value;

	SegTreeNode(int length) : l(0), r(0), level(0), value(length) {}
	SegTreeNode(int length, int index) : l(0), r(0), level(0), value(length, index) {}
};

typedef SegTreeNode * ptr_t;

class SegTree {
	ptr_t root;
	int width;
	
	void createEmpty(ptr_t & v, int l, int r) {
		int length = r - l + 1;
		if(v == 0) {
			v = new SegTreeNode(length, l);
		}
	}
	
	void add(ptr_t & v, int l, int r, int x) {
		int length = r - l + 1;
		if(v == 0) {
			v = new SegTreeNode(length);
		}
		if(l == r) {
			v->level += 1;
			v->value = EmptySegment(length);
		} else {
			int m = (l + r) / 2;
			if(x <= m)
				add(v->l, l, m, x);
			else
				add(v->r, m + 1, r, x);
			createEmpty(v->l, l, m);
			createEmpty(v->r, m + 1, r);
			v->value = EmptySegment::merge(v->l->value, v->r->value);
			//cerr << "ADD AFTER: " << l << " " << r << " ";
			//v->value.print();
			//cerr << endl;
		}
	}
	
	void remove(ptr_t & v, int l, int r, int x) {
		int length = r - l + 1;
		assert(v != 0);
		if(l == r) {
			v->level -= 1;
			if(v->level == 0)
				v->value = EmptySegment(length, l);
		} else {
			int m = (l + r) / 2;
			if(x <= m)
				remove(v->l, l, m, x);
			else
				remove(v->r, m + 1, r, x);
			assert(v->l != 0 && v->r != 0);
			v->value = EmptySegment::merge(v->l->value, v->r->value);
		}
	}
	
	void goDown(ptr_t & v, int l, int r, int squareSize, int & result) {
		int length = r - l + 1;
		
		if(length < squareSize) return;
		
		//cerr << "GO DOWN " << l << " " << r << " " << squareSize << " " << result << endl;
		
		int m = (l + r) / 2;
		createEmpty(v, l, r);
		
		if(v->value.result >= squareSize) {
			result = min(result, v->value.resultIndex);
		}
		
		if(l != r) {		
			createEmpty(v->l, l, m);
			
			if(v->l->value.result >= squareSize) {
				goDown(v->l, l, m, squareSize, result);
			} else {
				goDown(v->r, m + 1, r, squareSize, result);
			}
		}
	}

public:
	SegTree(int width) : root(0), width(width) {}


	void add(int x) {
		--x;
		if(x < 0) return;
		//cerr << "ST ADD: " << x << endl;
		assert(x < width);
		add(root, 0, width - 1, x);
	}
	
	void remove(int x) {
		--x;
		if(x < 0) return;
		//cerr << "ST REMOVE: " << x << endl;
		assert(x < width);
		remove(root, 0, width - 1, x);
	}
	
	int get(int squareSize) {
		//cerr << "ST GET: " << squareSize << endl;
		//squareSize;
		assert(squareSize >= 2);
		--squareSize;
		const int inf = 1e9 + 100;
		int resultIndex = inf;
		goDown(root, 0, width - 1, squareSize, resultIndex);
		return resultIndex == inf ? -1 : resultIndex;
	}
};

ll is_close(pair < int, int > p) {
	ll dx = p.first, dy = p.second;
	return dx * dx + dy * dy;
}

void relaxResult(result_t & target, result_t from) {
	target = min(target, from);
/*	if(target.first > from.first) {
		target = from;
	} else {
		assert(target.first == from.first);
		if(is_close(target.second) > is_close(from.second)) {
			target.second = from.second;
		} else if(is_close(target.second) == is_close(from.second)) {
			target = min(target, from);
		}
	}*/
}

result_t canPlace(int height, int width, int treesCount, const vector < tree_t > & trees, int squareSize) {
	//cerr << "CHECK: " << squareSize << " " << width << endl;

	result_t result = NONE;
	
	queue < event_t > removeEvents;
	SegTree st(width + 1);
	
	st.add(0);
	st.add(width);
	
	int cursor = treesCount - 1;
	
	while(cursor >= 0) {
		int x = trees[cursor].first;
		
		//cerr << "WTF: " << cursor << " " << x << " " << trees[cursor].second << endl;
		
		// remove old
		{
			while(!removeEvents.empty() && removeEvents.front().first >= x) {
				event_t event = removeEvents.front();
				removeEvents.pop();
				int y = event.second;
				st.remove(y);
			}
		}
		
		// update
		if(x + squareSize <= height) {
			int y = st.get(squareSize);
			if(y != -1) {
				//cerr << "UPDATE NOT LAST: " << x << " " << y << " " << cursor << endl << endl;
				result_t newResult = make_pair(squareSize, make_pair(x, y));
				relaxResult(result, newResult);
			}
		}
		
		// add new
		{
			while(cursor >= 0 && trees[cursor].first == x) {
				int y = trees[cursor].second;
				st.add(y);
				removeEvents.push(make_pair(x - squareSize, y));
				--cursor;
			}
		}
	}
	
	// x == 0
	{
		int x = 0;
		
		// remove old
		{
			while(!removeEvents.empty() && removeEvents.front().first >= x) {
				event_t event = removeEvents.front();
				removeEvents.pop();
				int y = event.second;
				st.remove(y);
			}
		}
		
		// update
		{
			int y = st.get(squareSize);
			if(y != -1) {
				//cerr << "UPDATE LAST: " << x << " " << y << endl;
				result_t newResult = make_pair(squareSize, make_pair(x, y));
				relaxResult(result, newResult);
			}
		}
	}
	
	return result;
}

int main() {
	ios_base::sync_with_stdio(false);
	
	int height, width;
	cin >> height >> width;
	
	if(height == 0 || width == 0) {
		cout << "0" << endl;
		cout << "0 0" << endl;
		return 0;
	}
	
	int treesCount;
	cin >> treesCount;
	vector < tree_t > trees(treesCount);
	for(int i = 0; i < treesCount; i++) {
		cin >> trees[i].first >> trees[i].second;
	}
	
	sort(all(trees));
	//reverse(all(trees));
	
	//for(int i = 0; i < treesCount; i++) {
	//	cout << trees[i].first << " " << trees[i].second << endl;
	//}
	
	int maxSquareSize = min(height, width);
	
	int l = 1, r = maxSquareSize + 1;
	while(r - l > 1) {
		int m = (l + r) / 2;
		if(canPlace(height, width, treesCount, trees, m) != NONE)
			l = m;
		else
			r = m;
	}
	
	if(l == 1) {
		cout << "1" << endl;
		cout << "0 0" << endl;
	} else {
		result_t result = canPlace(height, width, treesCount, trees, l);
		assert(result != NONE);
		cout << result.first << endl;
		cout << result.second.first << " " << result.second.second << endl;
	}
		
	return 0;
}