#include <algorithm>
#include <iostream>
#include <cmath>
using namespace std;

const int K = 400;
const int NK = 1200;

int n;
vector<int> a;

struct Segment
{
    Segment(int left, int right) :
        left(left),
        right(right)
    {
        int len = right - left + 1;
        values.resize(len);
        for (int pos = left; pos <= right; ++pos) {
            values[pos - left] = a[pos];
        }
        sorted_values.assign(values.begin(), values.end());
        sort(sorted_values.begin(), sorted_values.end());
    }

    int left;
    int right;
    vector<int> values;
    vector<int> sorted_values;

    int GetLowerOrEqualThan(int x)
    {
        if (sorted_values[0] > x) {
            return 0;
        }
        if (sorted_values.back() <= x) {
            return sorted_values.size();
        }
        int pos = 0;
        int bl = 0;
        int br = sorted_values.size() - 1;
        while (bl <= br) {
            int bm = (bl + br) >> 1;

            if (sorted_values[bm] <= x) {
                pos = bm;
                bl = bm + 1;
            } else {
                br = bm - 1;
            }
        }
        return pos + 1;
    }

    int Size() const
    {
        return values.size();
    }
};

ostream& operator << (ostream& output_stream, const Segment& seg)
{
    output_stream << "[";
    for (int i = 0; i < seg.Size(); ++i) {
        output_stream << seg.values[i];
        if (i + 1 < seg.Size()) {
            output_stream << " ";
        }
    }
    output_stream << "]";
    return output_stream;
}

vector<Segment*> segs;


void ShowSystem()
{
    for (int i = 0; i < (int) segs.size(); ++i) {
        cout << (*segs[i]) << endl;
    }
}

void Build()
{
    segs.clear();
    for (int i = 0; i * K < (int) a.size(); ++i) {
        segs.push_back(new Segment(i * K, min((int) a.size() - 1, (i + 1) * K - 1)));
    }
    n = a.size();
}

void Rebuild()
{
    a.resize(n);
    int temp_n = 0;
    for (int i = 0; i < (int) segs.size(); ++i) {
        for (int j = 0; j < segs[i]->Size(); ++j) {
            a[temp_n] = segs[i]->values[j];
            ++temp_n;
        }
    }
    Build();
}

int Split(int x)
{
    int cur_pos = 0;
    for (int i = 0; i < (int) segs.size(); ++i) {
        if (x == cur_pos) {
            return i;
        }
        int new_cur_pos = cur_pos + segs[i]->Size();
        if (x >= cur_pos && x < new_cur_pos) {
            int left = segs[i]->left;
            int right = segs[i]->right;
            int dx = x - cur_pos;
            segs[i] = new Segment(left, left + dx - 1);
            segs.insert(segs.begin() + i + 1, new Segment(left + dx, right));
            return i + 1;
        }
        cur_pos = new_cur_pos;
    }
}

bool NeedRebuild()
{
    return segs.size() > NK;
}

void Insert(int pos, int value)
{
    a.push_back(value);
    if (pos == n) {
        segs.push_back(new Segment(a.size() - 1, a.size() - 1));
    } else {
        int j = Split(pos);
        segs.insert(segs.begin() + j, new Segment(a.size() - 1, a.size() - 1));
    }
    ++n;
}

void Erase(int pos)
{
    int j = Split(pos);
    if (pos + 1 < n) {
        Split(pos + 1);
    }
    segs.erase(segs.begin() + j);
    --n;
}

int Get(int left, int right, int value)
{
    int l = Split(left);
    int r = Split(right + 1);
    int result = 0;
    for (int i = l; i < r; ++i) {
        result += segs[i]->GetLowerOrEqualThan(value);
    }
    return result;
}


int main()
{
    ios_base::sync_with_stdio(false);

    int temp_n;
    cin >> temp_n;
    a.resize(temp_n);
    for (int i = 0; i < temp_n; ++i) {
        cin >> a[i];
    }

    Build();

    char query_type;
    while (cin >> query_type) {
        if (query_type == '-') {
            int pos;
            cin >> pos;
            Erase(pos);
        } else if (query_type == '+') {
            int pos;
            int value;
            cin >> pos >> value;
            Insert(pos, value);
        } else if (query_type == '?') {
            int left;
            int right;
            int value;
            cin >> left >> right >> value;
            cout << Get(left, right, value) << '\n';
        }
        if (NeedRebuild()) {
            Rebuild();
        }
    }


    return 0;
}
