#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cassert>
#include <ctime>
#include <cstring>
#include <algorithm>
#include <iostream>
#include <set>
#include <map>
#include <vector>
#include <queue>
#include <deque>
using namespace std;
#define pb push_back
#define mp make_pair
#define sc second
#define fs first
#define int64 long long
#define ldb long double
#define sz(a) ((int) (a).size())
const int N = (int) 2e5 + 10;
const int limit = 500;
const int blocks_cnt = (2 * N) / limit + 10;
int cur_free, n, cur_blocks_cnt, blocks[blocks_cnt], next_free_block[blocks_cnt], x, y, z;
vector<int> data[blocks_cnt], elems[blocks_cnt];
char c;

int create_block() {
	int block = cur_free;
	cur_free = next_free_block[cur_free];
	return block;
}

void free_block(int block) {
	next_free_block[block] = cur_free;
	cur_free = block;
}

void merge_blocks(int pos) {
	//merge blocks[pos] and blocks[pos + 1]
	int i = blocks[pos], j = blocks[pos + 1];
	vector<int> ndata(sz(data[i]) + sz(data[j]));
	merge(data[i].begin(), data[i].end(), data[j].begin(), data[j].end(), ndata.begin());
	data[i] = ndata, data[j].clear();
	elems[i].insert(elems[i].end(), elems[j].begin(), elems[j].end()), elems[j].clear();
	free_block(j);
	for (int tmp = pos + 1; tmp + 1 < cur_blocks_cnt; ++tmp)
		blocks[tmp] = blocks[tmp + 1];
	cur_blocks_cnt--;
}

void split_block(int pos) {
	//split blocks[pos]
	int i = blocks[pos], j = create_block();
	int size = sz(elems[i]) / 2;
	elems[j] = data[j] = vector<int>(elems[i].begin() + size, elems[i].end());
	elems[i].resize(size), data[i] = elems[i];
	sort(data[i].begin(), data[i].end());
	sort(data[j].begin(), data[j].end());
	for (int tmp = cur_blocks_cnt - 1; tmp > pos; --tmp)
		blocks[tmp + 1] = blocks[tmp];
	blocks[pos + 1] = j;
	cur_blocks_cnt++;
}

int get(int x, int y) {
	if (x == 0) return 0;
	int res = 0;
	for (int tmp = 0; tmp < cur_blocks_cnt; ++tmp) {
		int i = blocks[tmp];
		if (sz(elems[i]) <= x) {
			res += upper_bound(data[i].begin(), data[i].end(), y) - data[i].begin();
			x -= sz(elems[i]);
		} else {
			for (int j = 0; j < x; ++j)
				res += (int) (elems[i][j] <= y);
			break;
		}
	}
	return res;
}

void add_element(int x, int y) {
	for (int tmp = 0; tmp < cur_blocks_cnt; ++tmp) {
		int i = blocks[tmp];
		if (sz(elems[i]) < x)
			x -= sz(elems[i]);
		else {
			elems[i].insert(elems[i].begin() + x, y);
			data[i].pb(y);
			for (int j = sz(data[i]) - 1; (j > 0) && (data[i][j] < data[i][j - 1]); --j)
				swap(data[i][j], data[i][j - 1]);
			if (sz(data[i]) > 2 * limit)
				split_block(tmp);
			return;
		}
	}
	
	assert(false);
}

void remove_element(int x) {
	for (int tmp = 0; tmp < cur_blocks_cnt; ++tmp) {
		int i = blocks[tmp];
		if (sz(elems[i]) <= x)
			x -= sz(elems[i]);
		else {
			int value = elems[i][x];
			elems[i].erase(elems[i].begin() + x);
			data[i].erase(find(data[i].begin(), data[i].end(), value));
			if ((tmp > 0) && (sz(data[blocks[tmp - 1]]) + sz(data[blocks[tmp]]) <= 2 * limit))
				merge_blocks(tmp - 1);
			return;
		}
	}
	assert(false);
}

int rnd() {
	return abs((rand() << 16) | rand());
}

int main() {
	/*assert(freopen("kthstat.in", "w", stdout));
	const int total = 100000;
	printf("%d\n", total);
	for (int i = 0; i < total; ++i)
		printf("%d ", rand());
	printf("\n");
	int size = total;
	for (int i = 0; i < total; ++i) {
		int t = rand() % 2;
		if (size == 0) t = 0;
		if (t == 0) {
			printf("+ %d %d\n", 0, rand());
			size++;
		} else if (t == 1) {
			printf("- %d\n", 0);
			size--;
		} else {
			int l = rnd() % size, r = rnd() % size;
			printf("? %d %d %d\n", min(l, r), max(l, r), rand());
		}
	}
	return 0;*/
	double start = clock();
	assert(freopen("kthstat.in", "r", stdin));
	assert(freopen("kthstat.out", "w", stdout));
	for (int i = 0; i + 1 < blocks_cnt; ++i)
		next_free_block[i] = i + 1;
	scanf("%d", &n);
	for (int i = 0; i < n; ++i) {
		scanf("%d", &x);
		int bnum = i / limit;
		elems[bnum].pb(x);
		cur_blocks_cnt = bnum + 1;
	}
	for (int i = 0; i < cur_blocks_cnt; ++i) {
		data[i] = elems[i];
		sort(data[i].begin(), data[i].end());
	}
	for (int i = 0; i < cur_blocks_cnt; ++i)
		blocks[i] = i;
	cur_free = cur_blocks_cnt;
	int it = 0;
	while (scanf(" %c", &c) == 1) {
		if (c == '+') {
			scanf("%d%d", &x, &y);
			add_element(x, y);
			n++;
		} else if (c == '-') {
			scanf("%d", &x);
			remove_element(x);
			n--;
		} else if (c == '?') {
			scanf("%d%d%d", &x, &y, &z);
			printf("%d\n", get(y + 1, z) - get(x, z));
		} else break;
		/*int sum = 0;
		for (int i = 0; i < cur_blocks_cnt; ++i)
			sum += sz(elems[blocks[i]]);
		assert(sum == n);
		assert(cur_blocks_cnt < blocks_cnt);*/
	}
	//fprintf(stderr, "%.2lf\n", (clock() - start) / CLOCKS_PER_SEC);
	return 0;
}