#include <iostream>
#include <algorithm>
#include <string>
#include <vector>
#include <set>
#include <map>
#include <queue>
#include <bitset>
#include <sstream>
#include <cstdio>
#include <cstring>
#include <cmath>
#include <climits>
#include <cctype>
#include <cstdlib>
using namespace std;

typedef long long LL;
typedef LL ll;

typedef pair<int, int> pii;
const int kInf = 1000 * 1000 * 1000;

struct Segment {
  vector<int> values;
  vector<int> sorted;

  Segment() {}
  Segment(int x) {
    values.push_back(x);
    sorted.push_back(x);
  }

  int get(int x) {
    return upper_bound(sorted.begin(), sorted.end(), x) - sorted.begin();
  }
};

vector<int> values;
vector<Segment*> T;
int K = 100;
int split(int i) {
  int pos = 0;
  while (pos < T.size() && T[pos]->values.size() <= i) {
    i -= T[pos]->values.size();
    pos++;
  }
  if (pos == T.size() || i == 0) {
    return pos;
  }
  Segment* first = T[pos];
  Segment* second = new Segment;
  T.insert(T.begin() + pos + 1, second);
  second->values = vector<int>(first->values.begin() + i, first->values.end());
  first->values.resize(i);
  first->sorted = first->values;
  sort(first->sorted.begin(), first->sorted.end());
  second->sorted = second->values;
  sort(second->sorted.begin(), second->sorted.end());
  return pos + 1;
}
void build() {
  int size = (values.size() + K - 1) / K;
  int cur = 0;
  while (cur < values.size()) {
    int next = min(cur + size, (int) values.size());
    Segment* seg = new Segment;
    seg->values = vector<int>(values.begin() + cur, values.begin() + next);
    seg->sorted = seg->values;
    sort(seg->sorted.begin(), seg->sorted.end());
    T.push_back(seg);
    cur = next;
  }
}
void rebuild() {
  int length = 0;
  for (int i = 0; i < T.size(); i++) {
    length += T[i]->values.size();
  }
  values.resize(length);
  int pos = 0;
  for (int i = 0; i < T.size(); i++) {
    copy(T[i]->values.begin(), T[i]->values.end(), values.begin() + pos);
    pos += T[i]->values.size();
  }
  T.clear();
  build();
}

void insert(int i, int x) {
  int pos = split(i);
  T.insert(T.begin() + pos, new Segment(x));
  if (T.size() > K * 3) {
    rebuild();
  }
}
void erase(int i) {
  int pos = split(i);
  split(i + 1);
  T.erase(T.begin() + pos);
  if (T.size() > K * 3) {
    rebuild();
  }
}
int query(int l, int r, int x) {
  l = split(l);
  r = split(r + 1);
  int res = 0;
  while (l < r) {
    res += T[l++]->get(x);
  }
  if (T.size() > K * 3) {
    rebuild();
  }
  return res;
}

int main() {
  int n;
  scanf("%d", &n);
  values.resize(n);
  for (int i = 0; i < n; i++) {
    scanf("%d", &values[i]);
  }
  build();
  char str[8];
  while (scanf("%s", str) == 1) {
    if (str[0] == '+') {
      int i, x;
      scanf("%d%d", &i, &x);
      insert(i, x);
    } else if (str[0] == '-') {
      int i;
      scanf("%d", &i);
      erase(i);
    } else if (str[0] == '?') {
      int l, r, x;
      scanf("%d%d%d", &l, &r, &x);
      printf("%d\n", query(l, r, x));
    }
  }

  return 0;
}
