#include <iostream>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <vector>
#include <string>
#include <map>
#include <set>
#include <algorithm>

using namespace std;

#define fs first
#define sc second

#define pb push_back
#define mp make_pair

#define forn(i, n) for(int i = 0; i < (int)n; ++i)
#define all(x) (x).begin(), (x).end()

const int MAXN = 100 * 1000 + 5, K = 600;
int a[2 * MAXN], n;

struct Segment {
    vector<int> vec;
    vector< pair<int, int> > sorted;

    Segment(int l, int r) {
        for(int i = l; i < r; i++) {
            vec.push_back(a[i]);
            sorted.push_back(make_pair(a[i], i));
        }
        sort(sorted.begin(), sorted.end());
    }

    Segment(Segment *seg, int l, int r) {
        for(int i = l; i < r; i++)
            vec.push_back(seg->vec[i]);
        for(size_t i = 0; i < seg->vec.size(); i++)
            if(l <= seg->sorted[i].second && seg->sorted[i].second < r)
                sorted.push_back(make_pair(seg->sorted[i].first, seg->sorted[i].second - l));
    }

    int get(int x) const {
        return upper_bound(sorted.begin(), sorted.end(), make_pair(x + 1, 0)) - sorted.begin();
    }
};

vector<Segment *> t;

int split(int pos) {
    for(size_t i = 0; i < t.size(); i++) {
        if(pos < (int)t[i]->vec.size()) {
            if(pos == 0)
                return i;
            Segment *l = new Segment(t[i], 0, pos), *r = new Segment(t[i], pos, t[i]->vec.size());
            delete t[i];
            t.erase(t.begin() + i);
            t.insert(t.begin() + i, r);
            t.insert(t.begin() + i, l);
            return i + 1;
        }
        pos -= (int)t[i]->vec.size();
    }
    return t.size();
}

void add(int pos, int x) {
    a[n++] = x;
    int k = split(pos);
    t.insert(t.begin() + k, new Segment(n - 1, n));
}

void del(int pos) {
    int k = split(pos);
    split(pos + 1);
    delete t[k];
    t.erase(t.begin() + k);
}

int get(int l, int r, int x) {
    l = split(l);
    r = split(r + 1);
    int res = 0;
    while(l < r) {
        res += t[l]->get(x);
        l++;
    }
    return res;
}

void build() {
    for(int l = 0; l < n; l += K) {
        int r = min(l + K, n);
        t.push_back(new Segment(l, r));
    }
}

void mer(Segment *l, Segment *r) {
    for(size_t i = 0; i < r->vec.size(); i++)
        l->vec.push_back(r->vec[i]);
    for(size_t i = 0; i < r->sorted.size(); i++)
        r->sorted[i].second += l->sorted.size();
    vector< pair<int, int> > tmp(l->sorted.size() + r->sorted.size());
    merge(l->sorted.begin(), l->sorted.end(), r->sorted.begin(), r->sorted.end(), tmp.begin());
    l->sorted.clear();
    for(size_t i = 0; i < tmp.size(); i++)
        l->sorted.push_back(tmp[i]);
    delete r;
}

void rebuild() {
    vector<Segment *> newt;
    for(size_t i = 0; i < t.size();) {
        size_t j = i + 1;
        while(j < t.size() && t[i]->vec.size() + t[j]->vec.size() <= K) {
            mer(t[i], t[j]);
            j++;
        }
        if(j < t.size() && t[i]->vec.size() < K) {
            Segment *l = new Segment(t[j], 0, K - (int)t[i]->vec.size()), *r = new Segment(t[j], 0, (int)t[j]->vec.size() - K + (int)t[i]->vec.size());
            mer(t[i], l);
            delete t[j];
            t[j] = r;
        }
        newt.push_back(t[i]);
        i = j;
    }
    t.clear();
    for(size_t i = 0; i < newt.size(); i++)
        t.push_back(newt[i]);
}

int main() {
    ios_base::sync_with_stdio(false);
    cin >> n;
    for(int i = 0; i < n; i++)
        cin >> a[i];
    build();
    char q;
    int m = 0;
    while(cin >> q) {
        if(q == '+') {
            int i, x;
            cin >> i >> x;
            add(i, x);
        }
        else if(q == '-') {
            int i;
            cin >> i;
            del(i);
        }
        else {
            int l, r, x;
            cin >> l >> r >> x;
            cout << get(l, r, x) << '\n';
        }
        m++;
        if(!(m % K))
            rebuild();
    }
    return 0;
}
