#include <iostream>
#include <vector>
#include <climits>
#include <algorithm>
#include <cmath>
#include <complex>
#include <iomanip>

using namespace std;

typedef complex<double> V;

typedef pair<V, V> Wall;

vector<Wall> oldwalls;
vector<Wall> walls;

bool ccw(V a, V b, V c) {
	return ((c - a) * conj(b - a)).imag() > 0;
}

double hit(V a, V b, V c, V d) {
	V u = (c - a) / (b - a);
	V v = (d - a) / (b - a);
	return (u.real() * v.imag() - u.imag() * v.real()) / (v.imag() - u.imag());
}

bool hits(V dir, V a, V b) {
	return ccw(0, dir, a) != ccw(0, dir, b) && hit(0, dir, a, b) > 0.0;
}

struct P {
	Wall wall;
	double endangle;
};

vector<double> angles;
const int N = 1 << 18;
const double PI = 4 * atan(1);
vector<P> T[2 * N];

bool f(P a, P b) { return ccw(0, a.wall.second - a.wall.first, b.wall.second - b.wall.first); }

void construct() {
	for(Wall wall : walls) oldwalls.push_back(wall);
	walls.clear();
	
	for(int i = 0; i < 2 * N; ++i) T[i].clear();
	
	vector<pair<double, int>> inds;
	int wallind = 0;
	for(int walli = 0; walli < oldwalls.size(); ++walli) {
		Wall wall = oldwalls[walli];
		double a = atan2(wall.first.imag(), wall.first.real());
		double b = atan2(wall.second.imag(), wall.second.real());
		if(a > b) {
			inds.emplace_back(a, wallind + 1);
			inds.emplace_back(PI, -wallind - 1);
			walls.push_back(wall);
			++wallind;
			inds.emplace_back(-PI, wallind + 1);
			inds.emplace_back(b, -wallind - 1);
			walls.push_back(wall);
			++wallind;
		} else {
			inds.emplace_back(a, wallind + 1);
			inds.emplace_back(b, -wallind - 1);
			walls.push_back(wall);
			++wallind;
		}
	}
	
	sort(inds.begin(), inds.end());
	
	angles.clear();
	vector<pair<int, int>> wallbounds(walls.size());
	for(int indi = 0; indi < inds.size(); ++indi) {
		angles.push_back(inds[indi].first);
		bool start = inds[indi].second > 0;
		int wall = abs(inds[indi].second) - 1;
		if(start) {
			wallbounds[wall].first = indi;
		} else {
			wallbounds[wall].second = indi;
		}
	}
	
	for(int walli = 0; walli < walls.size(); ++walli) {
		int a = N + wallbounds[walli].first;
		int b = N + wallbounds[walli].second - 1;
		
/*		cout << "Wall " << walls[walli].first << " - " << walls[walli].second << "\n";
		cout << " angles " << angles[a - N] << " - " << angles[b + 1 - N] << "\n";
	*/	
		while(a <= b) {
			if(a & 1) {
				T[a++].push_back(P{walls[walli], 0});
			}
			if(!(b & 1)) {
				T[b--].push_back(P{walls[walli], 0});
			}
			a /= 2;
			b /= 2;
		}
	}
	
	for(int v = 1; v <= 2 * N; ++v) {
		sort(T[v].begin(), T[v].end(), f);
	}
	
	walls.clear();
}

int main() {
	V perturb[2];
	double ang = 1e-12;
	perturb[0] = exp(V(0, ang));
	perturb[1] = exp(V(0, -ang));

	while(true) {
		if(walls.size() > 100) construct();
		string type;
		cin >> type;
		if(type == "end") break;
		if(type == "shot") {
			double vx, vy;
			cin >> vx >> vy;
			V dirb(vx, vy);
			double ret = 1.0 / 0.0;
			for(int di = 0; di <= 1; ++di) {
				V dir = dirb * perturb[di];
				double angle = atan2(dir.imag(), dir.real());
				for(Wall wall : walls) {
					if(hits(dir, wall.first, wall.second)) ret = min(ret, hit(0, dir, wall.first, wall.second));
				}
				if(!angles.empty()) {
					int v = lower_bound(angles.begin(), angles.end(), angle) - angles.begin();
					if(v != 0) --v;
					v += N;
					while(v != 0) {
						for(P p : T[v]) {
							if(hits(dir, p.wall.first, p.wall.second)) ret = min(ret, hit(0, dir, p.wall.first, p.wall.second));
						}
						v /= 2;
					}
				}
			}
			if(isfinite(ret)) {
				cout << setprecision(20) << ret << "\n";
			} else {
				cout << "Infinite\n";
			}
		} else {
			double x1, y1, x2, y2;
			cin >> x1 >> y1 >> x2 >> y2;
			V a(x1, y1);
			V b(x2, y2);
			if(!ccw(0, a, b)) swap(a, b);
			walls.emplace_back(a, b);
		}
	}
	
	return 0;
}
