#if 1
#include <functional>
#include <algorithm>
#include <iostream>
#include <iterator>
#include <iomanip>
#include <sstream>
#include <numeric>
#include <string>
#include <vector>
#include <bitset>
#include <queue>
#include <stack>
#include <map>
#include <set>
#include <cstdlib>
#include <cassert>
#include <cstring>
#include <cstdio>
#include <cmath>
#include <ctime>
#include <list>
 
using namespace std;
 
typedef long long LL;
typedef long double LD;
typedef pair<int, int> pii;
 
const LD eps = 1e-9;
const LD pi = acos(-1.0);
const LL inf = 1e+9;
 
#define mp make_pair
#define pb push_back
#define X first
#define Y second
 
#define dbg(x) { cerr << #x << " = " << x << endl; }
 
// extended template
#pragma comment(linker, "/STACK:36777216")
typedef vector<int> vi;
typedef vector<vi> vvi;
 
#define forn(i, n) for (int i = 0; i < n; ++i)
#define all(a) (a).begin(), (a).end()
#define rall(a) (a).rbegin(), (a).rend()
 
template<typename T> istream & operator >> (istream &, vector<T> &);
template<typename T> ostream & operator << (ostream &, const vector<T> &);
 
#define START clock_t _clock = clock();
#define END cerr << endl << "time: " << (clock() - _clock) / LD(CLOCKS_PER_SEC) << endl;
 
#define NAME "problem"

class segm_t;
typedef segm_t * psegm;

class segm_t
{
public:
    vi a, b;
    segm_t(int x) : a(vi(1, x)), b(vi(1, x)) {}
    segm_t(const vi &a) : a(a), b(a) { sort(all(b)); }
    int query(int x)
    {
        return upper_bound(all(b), x) - b.begin();
    }
    int len() const { return a.size(); }
};

vector<psegm> t;

void build(const vi &a)
{
    const int n = a.size();
    const int k = int(sqrt(LD(n)) - eps + 1);

    forn(i, t.size())
        delete t[i];
    
    t.clear();
    for (int i = 0; i < n; i += k)
    {
        int l = i;
        int r = min(i + k, n);
        t.push_back(new segm_t(vi(a.begin() + l, a.begin() + r)));
    }
}

int split(int x)
{
    int l = 0, r = 0;
    forn(i, t.size())
    {
        r += t[i]->len();
        if (l <= x && x < r)
        {
            if (l == x)
                return i;

            const vi &vals = t[i]->a;
            int pos = x - l;
            psegm T1 = new segm_t(vi(vals.begin(), vals.begin() + pos));
            psegm T2 = new segm_t(vi(vals.begin() + pos, vals.end()));
            t.erase(t.begin() + i);
            t.insert(t.begin() + i, T1);
            t.insert(t.begin() + i + 1, T2);
            return i + 1;
        }
        l = r;
    }
    return t.size();
}

void insert(int pos, int x)
{
    int idx = split(pos);
    t.insert(t.begin() + idx, new segm_t(x));
}

void erase(int pos)
{
    int idx = split(pos);
    split(pos + 1);
    t.erase(t.begin() + idx);
}

int query(int l, int r, int x)
{
    l = split(l);
    r = split(r + 1);
    int res = 0;
    for (int i = l; i < r; ++i)
        res += t[i]->query(x);
    return res;
}

void rebuild()
{
    vi a;
    forn(i, t.size())
        a.insert(a.end(), t[i]->a.begin(), t[i]->a.end());
    build(a);
}

char c[10];

void solve()
{
    int n; scanf("%d", &n);

    int cnt = n;

    vi a(n);
    forn(i, n)
        scanf("%d", &a[i]);
    build(a);

    
    while (~scanf("%s", c))
    {
        if (c[0] == '-')
        {
            int i; scanf("%d", &i); erase(i);
            --cnt;
        }
        else if (c[0] == '+')
        {
            int i, x; scanf("%d %d", &i, &x);
            insert(i, x);
            ++cnt;
        }
        else
        {
            int l, r, x; scanf("%d %d %d", &l, &r, &x);
            int res = query(l, r, x);
            printf("%d\n", res);
        }

        if (t.size() > 3 * sqrt(LD(cnt)))
            rebuild();
    }
}
 
int main()
{
    // freopen(NAME ".in", "r", stdin); freopen(NAME ".out", "w", stdout);
    // freopen("input.txt", "r", stdin); // freopen("output.txt", "w", stdout);
 
    solve();
 
    return 0;
}
/*******************************************
*******************************************/
 
template<typename T> istream & operator >> (istream &is, vector<T> &v)
{
    forn(i, v.size())
        is >> v[i];
    return is;
}
template<typename T> ostream & operator << (ostream &os, const vector<T> &v)
{
    forn(i, v.size())
        os << v[i] << " ";
    return os;
}
#endif