#include <iostream>
#include <fstream>
#include <vector>

using namespace std;

const int kParts = 300;

int main() {
    ios::sync_with_stdio(false);

    int N; cin >> N;

    vector<int> V(N);
    for (int i = 0; i < N; ++i)
        cin >> V[i];

    vector< vector<int> > numbers(kParts);
    vector< vector<int> > sorted(kParts);

    auto copyInV = [&]() {
        V.clear();
        V.reserve(N);
        for (int i = 0; i < kParts; ++i)
            for (auto &x: numbers[i])
                V.push_back(x);
    };

    auto rebuild = [&]() {
        for (int i = 0; i < N; ++i)
            numbers[i / kParts].push_back(V[i]);
        for (int i = 0; i < kParts; ++i) {
            sorted[i] = numbers[i];
            sort(sorted[i].begin(), sorted[i].end());
        }
    };

    auto erase = [&](int pos) {
        ++pos;
        --N;
        for (int i = 0; i < kParts; ++i) {
            int aux = numbers[i].size();
            if (aux < pos) {
                pos -= aux;
                continue;
            }

            int whom = numbers[i][pos - 1];
            sorted[i].erase(lower_bound(sorted[i].begin(), sorted[i].end(), whom));

            numbers[i].erase(numbers[i].begin() + pos - 1);
            break;
        }
    };

    auto insert = [&](int pos, int value) {
        ++N;
        for (int i = 0; i < kParts; ++i) {
            int aux = numbers[i].size();
            if (aux < pos) {
                pos -= aux;
                continue;
            }

            numbers[i].insert(numbers[i].begin() + pos, value);
            sorted[i].insert(lower_bound(sorted[i].begin(), sorted[i].end(), value), value);
        }
    };

    auto query = [&](int from, int to, int value) {
        int answer = 0;
        for (int i = 0; i < kParts; ++i) {
            if (to < 0)
                break;

            if (from < int(numbers[i].size())) {
                int rfrom = max(from, 0);
                int rto = min<int>(to, numbers[i].size() - 1);

                if (rto - rfrom + 1 == int(numbers[i].size())) {
                    answer += upper_bound(sorted[i].begin(), sorted[i].end(), value) - sorted[i].begin();
                } else {
                    for (int j = rfrom; j <= rto; ++j)
                        if (numbers[i][j] <= value)
                            ++answer;
                }
            }

            from -= numbers[i].size();
            to -= numbers[i].size();
        }
        return answer;
    };

    int ops = 0;
    char type;
    rebuild();
    while (cin >> type) {
        if (type == '-') {
            int x; cin >> x;
            erase(x);
        } else if (type == '+') {
            int x; int y;
            cin >> x >> y;
            insert(x, y);
        } else {
            int x, y, z;
            cin >> x >> y >> z;
            cout << query(x, y, z) << "\n";
        }
        if (++ops % kParts == 0) {
            copyInV();
            rebuild();
        }
    }
}
