#include <iostream>
#include <fstream>
#include <vector>
#include <algorithm>

using namespace std;

class SegmentTree {
  public:
    SegmentTree(int size):
            m_min(size * 4, 0),
            m_add(size * 4, 0),
            m_size(size) {
    }

    void update(int from, int to, int value) {
        if (from >= to)
            return;
        update(1, 0, m_size, from, to, value);
    }

    pair<int, int> query() {
        return query(1, 0, m_size);
    }

  private:
    void update(int node, int begin, int end, int from, int to, int value) {
        if (from <= begin && end <= to) {
            m_add[node] += value;
            return;
        }

        int mid = (begin + end) / 2;
        if (from < mid)
            update(node * 2, begin, mid, from, to, value);
        if (mid < to)
            update(node * 2 + 1, mid, end, from, to, value);

        m_min[node] = min(m_min[node * 2] + m_add[node * 2], m_min[node * 2 + 1] + m_add[node * 2 + 1]);
    }

    pair<int, int> query(int node, int begin, int end) {
        if (end - begin == 1)
            return make_pair(m_min[node] + m_add[node], begin);

        int mid = (begin + end) / 2;
        if (m_min[node * 2] + m_add[node * 2] == m_min[node]) {
            auto answer = query(node * 2, begin, mid);
            answer.first += m_add[node];
            return answer;
        }

        auto answer = query(node * 2 + 1, mid, end);
        answer.first += m_add[node];
        return answer;
    }

    vector<int> m_min, m_add;
    int m_size;
};

pair<int, int> find(vector< pair<int, int> > P, int side, int ox, int ) {
    int N = P.size();
    vector<int> Y;
    Y.reserve(N);
    for (int i = 0; i < N; ++i)
        Y.push_back(P[i].second);

    sort(Y.begin(), Y.end());
    Y.erase(unique(Y.begin(), Y.end()), Y.end());

    int M = Y.size();
    vector<int> badTo(M);
    for (int i = 0; i < M; ++i)
        badTo[i] = upper_bound(Y.begin(), Y.end(), Y[i] - side) - Y.begin();

    sort(P.begin(), P.end());
    SegmentTree S(M);
    S.update(badTo.back(), M, N);

    int index = 0;
    for (int i = 0; i < N;) {
        if (P[i].first + side > ox)
            break;
        while (index < N && P[index].first < P[i].first + side) {
            int to = lower_bound(Y.begin(), Y.end(), P[index].second) - Y.begin();
            int from = badTo[to];
            S.update(from, to, 1);
            ++index;
        }

        int j;
        for (j = i; j < N && P[j].first == P[i].first; ++j) {
            int to = lower_bound(Y.begin(), Y.end(), P[j].second) - Y.begin();
            int from = badTo[to];
            S.update(from, to, -1);
        }

        auto answer = S.query();
        if (answer.first == 0)
            return make_pair(P[i].first, Y[answer.second]);
        i = j;
    }
    return make_pair(-1, -1);
}

int main() {
    int X, Y; cin >> X >> Y;
    ios::sync_with_stdio(false);

    int N; cin >> N;

    vector< pair<int, int> > P(N);
    for (int i = 0; i < N; ++i)
        cin >> P[i].first >> P[i].second;
    P.push_back(make_pair(0, 0));
    P.push_back(make_pair(X, Y));

    int step = (1 << 30);
    int answer;
    for (answer = 0; step; step >>= 1) {
        answer += step;

        if (find(P, answer, X, Y).first == -1)
            answer -= step;
    }

    int x, y;
    tie(x, y) = find(P, answer, X, Y);

    cout << answer << "\n";
    cout << x << " " << y << "\n";
}
